﻿using System.Collections;
using System.Collections.Generic;
using System.Data;
using BBCode.Nodes;

namespace BBCode.Collections
{
    /// <summary>
    /// A collection of Nodes
    /// </summary>
    public class NodeCollection : ICollection<INode>
    {
        private readonly IDictionary<string, INode> _collection = new Dictionary<string, INode>();

        /// <summary>
        /// Initializes a new instance of the <see cref="NodeCollection"/> class.
        /// </summary>
        public NodeCollection()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="NodeCollection"/> class and adds the supplied items.
        /// </summary>
        /// <param name="items">The items to be added to the collection.</param>
        public NodeCollection(IEnumerable<INode> items)
        {
            AddRange(items);
        }

        #region ICollection<INode> Members

        /// <summary>
        /// Gets the enumerator.
        /// </summary>
        /// <returns></returns>
        public IEnumerator<INode> GetEnumerator()
        {
            return _collection.Values.GetEnumerator();
        }

        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary>
        /// <returns>
        /// An <see cref="T:System.Collections.IEnumerator"/> object that can be used to iterate through the collection.
        /// </returns>
        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        /// <summary>
        /// Adds the specified item.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Add(INode item)
        {
            //Check if key already present!
            if (_collection.ContainsKey(item.ID))
            {
                throw new DuplicateNameException("A INode with the same ID already exists in the collection.");
            }
            _collection.Add(item.ID, item);
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            _collection.Clear();
        }

        /// <summary>
        /// Determines whether the collection [contains] [the specified item].
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        ///   <c>true</c> if the collection [contains] [the specified item]; otherwise, <c>false</c>.
        /// </returns>
        public bool Contains(INode item)
        {
            return _collection.ContainsKey(item.ID);
        }

        /// <summary>
        /// Copies to.
        /// </summary>
        /// <param name="array">The array.</param>
        /// <param name="arrayIndex">Index of the array.</param>
        public void CopyTo(INode[] array, int arrayIndex)
        {
            _collection.Values.CopyTo(array, arrayIndex);
        }

        /// <summary>
        /// Removes the specified item.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns></returns>
        public bool Remove(INode item)
        {
            return _collection.Remove(item.ID);
        }

        /// <summary>
        /// Gets the count.
        /// </summary>
        public int Count
        {
            get { return _collection.Count; }
        }

        /// <summary>
        /// Gets a value indicating whether this instance is read only.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is read only; otherwise, <c>false</c>.
        /// </value>
        public bool IsReadOnly
        {
            get { return _collection.IsReadOnly; }
        }

        #endregion

        /// <summary>
        /// Adds the range.
        /// </summary>
        /// <param name="items">The items.</param>
        public void AddRange(IEnumerable<INode> items)
        {
            foreach (INode item in items)
            {
                Add(item);
            }
        }

        /// <summary>
        /// Removes the range.
        /// </summary>
        /// <param name="items">The items.</param>
        public void RemoveRange(IEnumerable<INode> items)
        {
            foreach (var item in items)
            {
                Remove(item);
            }
        }
    }
}